use crate::current_weather::CurrentWeather;

pub struct CurrentWeatherPrinter {
    current_weather: CurrentWeather,
    clean: bool,
}

impl CurrentWeatherPrinter {
    pub fn new(current_weather: CurrentWeather, clean: bool) -> CurrentWeatherPrinter {
        CurrentWeatherPrinter {
            current_weather,
            clean,
        }
    }
    pub fn extract_day(&self) -> String {
        if self.current_weather.is_day == 1 {
            self.parse_custom_data(&self.current_weather.is_day.to_string(), "Day")
        } else {
            self.parse_custom_data(&self.current_weather.is_day.to_string(), "Night")
        }
    }

    pub fn extract_temperature(&self) -> String {
        self.parse_simple_data(
            &self.current_weather.temperature.to_string(),
            "Temperature",
            Some("°C"),
        )
    }

    pub fn extract_wind_speed(&self) -> String {
        self.parse_simple_data(
            &self.current_weather.windspeed.to_string(),
            "Wind speed",
            Some(" km/h"),
        )
    }

    pub fn extract_wind_direction(&self) -> String {
        self.parse_simple_data(
            &self.current_weather.winddirection.to_string(),
            "Wind direction",
            Some("°"),
        )
    }

    fn parse_custom_data(&self, data: &str, custom: &str) -> String {
        if self.clean {
            format!("{data}")
        } else {
            format!("{custom}")
        }
    }

    fn parse_simple_data(&self, data: &str, descriptor: &str, end_text: Option<&str>) -> String {
        if self.clean {
            format!("{data}")
        } else {
            let end_text = end_text.unwrap_or("");
            format!("{descriptor}: {data}{end_text}")
        }
    }
}

#[cfg(test)]
mod tests {
    use super::*;

    #[test]
    fn clean_data() {
        let current_weather = CurrentWeather {
            is_day: 1,
            temperature: 12.5,
            windspeed: 7.0,
            winddirection: 90.0,
        };
        let printer = CurrentWeatherPrinter::new(current_weather, true);

        assert_eq!(printer.extract_day(), "1");
        assert_eq!(printer.extract_temperature(), "12.5");
        assert_eq!(printer.extract_wind_speed(), "7");
        assert_eq!(printer.extract_wind_direction(), "90");
    }

    #[test]
    fn full_data() {
        let current_weather = CurrentWeather {
            is_day: 0,
            temperature: 22.0,
            windspeed: 15.5,
            winddirection: 118.0,
        };
        let printer = CurrentWeatherPrinter::new(current_weather, false);

        assert_eq!(printer.extract_day(), "Night");
        assert_eq!(printer.extract_temperature(), "Temperature: 22°C");
        assert_eq!(printer.extract_wind_speed(), "Wind speed: 15.5 km/h");
        assert_eq!(printer.extract_wind_direction(), "Wind direction: 118°");
    }
}
